@@ -11,7 +11,7 @@ import com.unciv.models.ruleset.Belief
 import com.unciv.models.ruleset.BeliefType
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
-import com.unciv.ui.utils.extensions.toPercent
+import com.unciv.ui.components.extensions.toPercent
 import java.lang.Integer.max
 import java.lang.Integer.min
 import kotlin.random.Random
@@ -186,20 +186,24 @@ class ReligionManager : IsPartOfGameInfoSerialization {
         }
     }
 
-    private fun maxNumberOfReligions() = min(
-        civInfo.gameInfo.ruleset.religions.size,
-        civInfo.gameInfo.civilizations.count { it.isMajorCiv() } / 2 + 1
-    )
+    private fun maxNumberOfReligions(): Int {
+        val gameInfo = civInfo.gameInfo
+        val ruleset = gameInfo.ruleset
+        val multiplier = ruleset.modOptions.constants.religionLimitMultiplier
+        val base = ruleset.modOptions.constants.religionLimitBase
+        val civCount = gameInfo.civilizations.count { it.isMajorCiv() }
+        return min(ruleset.religions.size, base + (civCount * multiplier).toInt())
+    }
+
+    /** Calculates the number of religions that are already founded */
+    private fun foundedReligionsCount() = civInfo.gameInfo.civilizations.count {
+        it.religionManager.religion != null && it.religionManager.religionState >= ReligionState.Religion
+    }
 
     /** Calculates the amount of religions that can still be founded */
     fun remainingFoundableReligions(): Int {
-        val gameInfo = civInfo.gameInfo
-        val foundedReligionsCount = gameInfo.civilizations.count {
-            it.religionManager.religion != null && it.religionManager.religionState >= ReligionState.Religion
-        }
-
         // count the number of foundable religions left given defined ruleset religions and number of civs in game
-        val maxNumberOfAdditionalReligions = maxNumberOfReligions() - foundedReligionsCount
+        val maxNumberOfAdditionalReligions = maxNumberOfReligions() - foundedReligionsCount()
 
         val availableBeliefsToFound = min(
             numberOfBeliefsAvailable(BeliefType.Follower),
@@ -209,6 +213,34 @@ class ReligionManager : IsPartOfGameInfoSerialization {
         return min(maxNumberOfAdditionalReligions, availableBeliefsToFound)
     }
 
+    /** Get info breaking down the reasons behind the result of [remainingFoundableReligions] */
+    fun remainingFoundableReligionsBreakdown() = sequence {
+        val gameInfo = civInfo.gameInfo
+        val ruleset = gameInfo.ruleset
+        yield("Available religion symbols" to ruleset.religions.size)
+
+        val multiplier = ruleset.modOptions.constants.religionLimitMultiplier
+        val base = ruleset.modOptions.constants.religionLimitBase
+        val civCount = gameInfo.civilizations.count { it.isMajorCiv() }
+        val hideCivCount = civInfo.hideCivCount()
+        if (hideCivCount) {
+            val knownCivs = 1 + civInfo.getKnownCivs().count { it.isMajorCiv() }
+            val estimatedCivCount = (
+                    gameInfo.gameParameters.minNumberOfPlayers.coerceAtLeast(knownCivs) +
+                            gameInfo.gameParameters.maxNumberOfPlayers - 1
+                    ) / 2 + 1
+            val civsAndBase = base + (estimatedCivCount * multiplier).toInt()
+            yield("Estimated number of civilizations * [$multiplier] + [$base]" to civsAndBase)
+        } else {
+            val civsAndBase = base + (civCount * multiplier).toInt()
+            yield("Number of civilizations * [$multiplier] + [$base]" to civsAndBase)
+        }
+
+        yield("Religions already founded" to foundedReligionsCount())
+        yield("Available founder beliefs" to numberOfBeliefsAvailable(BeliefType.Founder))
+        yield("Available follower beliefs" to numberOfBeliefsAvailable(BeliefType.Follower))
+    }
+
     fun numberOfBeliefsAvailable(type: BeliefType): Int {
         val gameInfo = civInfo.gameInfo
         val numberOfBeliefs = if (type == BeliefType.Any) gameInfo.ruleset.beliefs.values.count()
@@ -405,7 +437,7 @@ class ReligionManager : IsPartOfGameInfoSerialization {
     fun maySpreadReligionAtAll(missionary: MapUnit): Boolean {
         if (!civInfo.isMajorCiv()) return false // Only major civs
         if (!civInfo.gameInfo.isReligionEnabled()) return false // No religion, no spreading
-        if (!missionary.canDoReligiousAction(Constants.spreadReligion)) return false
+        if (!missionary.canDoLimitedAction(Constants.spreadReligion)) return false
         return true
     }
 
