@@ -5,6 +5,8 @@ import com.badlogic.gdx.scenes.scene2d.ui.CheckBox
 import com.badlogic.gdx.scenes.scene2d.ui.Table
 import com.badlogic.gdx.scenes.scene2d.ui.TextField
 import com.badlogic.gdx.scenes.scene2d.ui.TextField.TextFieldFilter.DigitsOnlyFilter
+import com.badlogic.gdx.utils.Align
+import com.unciv.UncivGame
 import com.unciv.logic.map.MapGeneratedMainType
 import com.unciv.logic.map.MapParameters
 import com.unciv.logic.map.MapResources
@@ -16,6 +18,7 @@ import com.unciv.logic.map.mapgenerator.MapGenerationRandomness
 import com.unciv.ui.components.ExpanderTab
 import com.unciv.ui.components.UncivSlider
 import com.unciv.ui.components.UncivTextField
+import com.unciv.ui.components.WrappableLabel
 import com.unciv.ui.components.extensions.onChange
 import com.unciv.ui.components.extensions.onClick
 import com.unciv.ui.components.extensions.pad
@@ -274,6 +277,14 @@ class MapParametersTable(
                 mapParameters.mapResources = resourceSelectBox.selected.value
             }
 
+            if (forMapEditor) {
+                val comment = "This is used for painting resources, not in map generator steps:"
+                val expectedWidth = (UncivGame.Current.screen?.stage?.width ?: 1200f) * 0.4f
+                val label = WrappableLabel(comment, expectedWidth, Color.GOLD, 14)
+                label.setAlignment(Align.center)
+                label.wrap = true
+                add(label).colspan(2).row()
+            }
             add("{Resource Setting}:".toLabel()).left()
             add(resourceSelectBox).fillX().row()
         }
@@ -362,7 +373,7 @@ class MapParametersTable(
             return slider
         }
 
-        fun addTextButton(text: String, shouldAddToTable: Boolean = false, action: ((Boolean) -> Unit),) {
+        fun addTextButton(text: String, shouldAddToTable: Boolean = false, action: ((Boolean) -> Unit)) {
             val button = text.toTextButton()
             button.onClick { action.invoke(true) }
             if (shouldAddToTable)
@@ -378,8 +389,10 @@ class MapParametersTable(
         addSlider("Temperature shift", {mapParameters.temperatureShift}, -0.4f, 0.4f, 0.1f)
         { mapParameters.temperatureShift = it }
 
-        addSlider("Resource richness", {mapParameters.resourceRichness},0f, 0.5f)
-        { mapParameters.resourceRichness = it }
+        if (forMapEditor) {
+            addSlider("Resource richness", { mapParameters.resourceRichness }, 0f, 0.5f)
+            { mapParameters.resourceRichness = it }
+        }
 
         addSlider("Vegetation richness", {mapParameters.vegetationRichness}, 0f, 1f)
         { mapParameters.vegetationRichness = it }
