@@ -21,6 +21,7 @@ import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.removeConditionals
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.Fonts
@@ -336,9 +337,9 @@ object UnitActions {
             // Check _new_ resource requirements
             // Using Counter to aggregate is a bit exaggerated, but - respect the mad modder.
             val resourceRequirementsDelta = Counter<String>()
-            for ((resource, amount) in unit.baseUnit().getResourceRequirements())
+            for ((resource, amount) in unit.baseUnit().getResourceRequirementsPerTurn())
                 resourceRequirementsDelta.add(resource, -amount)
-            for ((resource, amount) in upgradedUnit.getResourceRequirements())
+            for ((resource, amount) in upgradedUnit.getResourceRequirementsPerTurn())
                 resourceRequirementsDelta.add(resource, amount)
             val newResourceRequirementsString = resourceRequirementsDelta.entries
                 .filter { it.value > 0 }
@@ -412,7 +413,7 @@ object UnitActions {
     }
 
     private fun addRepairAction(unit: MapUnit, actionList: ArrayList<UnitAction>) {
-        if (unit.currentTile.ruleset.tileImprovements[Constants.repair] == null) return
+        if (!unit.currentTile.ruleset.tileImprovements.containsKey(Constants.repair)) return
         if (!unit.cache.hasUniqueToBuildImprovements) return
         if (unit.isEmbarked()) return
         val tile = unit.getTile()
@@ -601,7 +602,7 @@ object UnitActions {
         if (isDamaged && !showingAdditionalActions) {
             actionList += UnitAction(UnitActionType.SleepUntilHealed,
                 action = { unit.action = UnitActionType.SleepUntilHealed.value }
-                    .takeIf { !unit.isSleepingUntilHealed() }
+                    .takeIf { !unit.isSleepingUntilHealed() && unit.canHealInCurrentTile() }
             )
         } else if (isDamaged || !showingAdditionalActions) {
             actionList += UnitAction(UnitActionType.Sleep,
@@ -675,9 +676,14 @@ object UnitActions {
                     && unique.conditionals.none { it.type == UniqueType.ConditionalTimedUnique }) continue
             if (usagesLeft(unit, unique)==0) continue
 
-            val unitAction = UnitAction(type = UnitActionType.TriggerUnique,
-                title = actionTextWithSideEffects(unique.text.removeConditionals(), unique, unit)
-                ){
+            val baseTitle = if (unique.isOfType(UniqueType.OneTimeEnterGoldenAgeTurns))
+                    unique.placeholderText.fillPlaceholders(
+                        unit.civ.goldenAges.calculateGoldenAgeLength(
+                            unique.params[0].toInt()).toString())
+                else unique.text.removeConditionals()
+            val title = actionTextWithSideEffects(baseTitle, unique, unit)
+
+            val unitAction = UnitAction(type = UnitActionType.TriggerUnique, title){
                 UniqueTriggerActivation.triggerUnitwideUnique(unique, unit)
                 activateSideEffects(unit, unique)
             }
@@ -758,7 +764,7 @@ object UnitActions {
         return null
     }
 
-    fun actionTextWithSideEffects(originalText:String, actionUnique: Unique, unit: MapUnit): String {
+    fun actionTextWithSideEffects(originalText: String, actionUnique: Unique, unit: MapUnit): String {
         val sideEffectString = getSideEffectString(unit, actionUnique)
         if (sideEffectString == "") return originalText
         else return "{$originalText} $sideEffectString"
